from dataclasses import dataclass, field
from typing import Any, List, Optional

import numpy as np

from mag_tools.model.justify_type import JustifyType
from mag_tools.utils.data.value_utils import ValueUtils
from mag_tools.utils.data.string_format import StringFormat


@dataclass
class TextFormat:
    """
    文本拼接的格式，用于将多个字符串拼接为一个文本时使用
    """
    number_per_line: Optional[int] = field(default=None, metadata={"description": "每行显示的数据个数"})
    justify_type: Optional[JustifyType] = field(default=JustifyType.LEFT, metadata={"description": "对齐方式"})
    at_header: Optional[str] = field(default='', metadata={"description": "句首添加的字符串"})
    decimal_places: Optional[int] = field(default=2, metadata={"description": "小数位数"})
    decimal_places_of_zero: Optional[int] = field(default=1, metadata={"description": "小数为0时的小数位数"})
    pad_length: Optional[int] = field(default=None, metadata={"description": "字段显示长度，为 None 表示各字段自行定义"})
    pad_char: Optional[str] = field(default=' ', metadata={"description": "填充占位符"})
    scientific: bool = field(default=False, metadata={"description": "是否使用科学计数法"})

    def __str__(self):
        """
        返回 TextFormat 实例的字符串表示。
        :return: TextFormat 实例的字符串表示。
        """
        return f"TextFormat({', '.join(f'{key}={value}' for key, value in vars(self).items())})"

    def to_lines(self, values: List[Any], max_length: int = None) -> List[str]:
        if max_length is None:
            max_length = self.pad_length

        lines = []
        for value in values:
            if isinstance(value, int) or isinstance(value, float):
                lines.append(ValueUtils.to_string(value, self.decimal_places, self.decimal_places_of_zero, self.scientific))
            else:
                lines.append(str(value))

        return StringFormat.pad_text(lines, max_length, self.justify_type, self.number_per_line, self.at_header)

    def to_blocks(self, values: List[List[Any]]) -> List[str]:
        max_len = max(len(str(item)) for sublist in values for item in sublist)
        lines = []
        for value in values:
            lines.extend(self.to_lines(value, max_len))
        return lines

    def array_1d_to_lines(self, array_1d: List[Any]) -> List[str]:
        """
        将一维数组转换为多行文本表示
        :param array_1d: 一维数组，包含任意类型的数值
        :return: 多行文本表示的数组
        """
        if array_1d is None:
            return []

        result = []
        count = 1
        current_value = array_1d[0]

        max_len = 2
        for i in range(1, len(array_1d)):
            if array_1d[i] == current_value:
                count += 1
            else:
                current_text = ValueUtils.to_string(current_value, self.decimal_places,
                                                    self.decimal_places_of_zero, self.scientific)
                txt = f"{count}*{current_text}" if count > 1 else f"{current_text}"
                max_len = max(max_len, len(txt))
                result.append(txt)

                current_value = array_1d[i]
                count = 1

        # 处理最后一组数据
        current_text = ValueUtils.to_string(current_value, self.decimal_places,
                                            self.decimal_places_of_zero, self.scientific)
        txt = f"{count}*{current_text}" if count > 1 else f"{current_text}"
        max_len = max(max_len, len(txt))
        result.append(txt)

        return self.to_lines(result, max_len)

    def array_2d_to_lines(self, array_2d: List[List[Any]]) -> List[str]:
        """
        生成数值块
        :param array_2d: 二维数组
        :return: 数值块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        """

        # 将其展平为一维数组后添加到列表中
        layer = np.array(array_2d).flatten()
        return self.array_1d_to_lines(layer.tolist())

    def array_3d_to_lines(self, array_3d: List[List[List[Any]]]) -> List[str]:
        """
        生成数值块
        :param array_3d: 三维数组
        :return: 数值块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        """

        # 遍历每一层并将其展平为一维数组后添加到列表中
        array_1d_sum = []
        num_layers = np.array(array_3d).shape[0]  # 层数
        for i in range(num_layers):
            array_2d = array_3d[i]
            array_1d = np.array(array_2d).flatten()
            array_1d_sum.extend(array_1d.tolist())
            # array_1d_sum.extend(ValueUtils.to_native(value) for value in array_1d)
        return self.array_1d_to_lines(array_1d_sum)



from typing import Any, List

import numpy as np

from mag_tools.bean.text_format import TextFormat
from mag_tools.utils.data.value_utils import ValueUtils


class ArrayUtils:
    @staticmethod
    def text_to_array_1d(text: str, data_type: type) -> List[Any]:
        """
        从一行文本中获取数值数组[]
        :param text: 文本，由 "数值" 或 "数值个数*数值"构成
        :param data_type: 数据类型
        :return: 数值数组
        """
        data = []

        for part in text.split():
            if '*' in part:
                count, value = part.split('*')
                data.extend([ValueUtils.to_value(value, data_type)] * int(count))
            else:
                data.append(ValueUtils.to_value(part, data_type))

        return data

    @staticmethod
    def lines_to_array_1d(lines: List[str], data_type: type) -> List[Any]:
        """
        从多行文本中获取数值数组[]
        :param lines: 文本段，每行由 "数值" 或 "数值个数*数值"构成
        :param data_type: 数据类型
        :return: 数值数组
        """
        array_1d = []
        for line in lines:
            array_1d.extend(ArrayUtils.text_to_array_1d(line, data_type))
        return array_1d

    @staticmethod
    def lines_to_array_2d(lines: List[str], nx: int, ny: int, data_type: type) -> List[
        List[Any]]:
        """
        从文本块获取数值列表
        :param lines: 文本块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        :param nx: 行数
        :param ny: 列数
        :param data_type: 数据类型
        :return: Values
        """
        array_1d = ArrayUtils.lines_to_array_1d(lines, data_type)
        return np.array(array_1d).reshape((nx, ny)).tolist()

    @staticmethod
    def lines_to_array_3d(block_lines: List[str], nx: int, ny: int, nz: int, data_type: type) -> List[List[List[Any]]]:
        """
        从文本块获取数值列表
        :param block_lines: 文本块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        :param nx: 行数
        :param ny: 列数
        :param nz: 层数
        :param data_type: 数据类型
        :return: Values
        """
        array_1d = ArrayUtils.lines_to_array_1d(block_lines, data_type)
        return np.array(array_1d).reshape((nx, ny, nz)).tolist()

    @staticmethod
    def assign_array_2d(array2, i1, i2, j1, j2, value):
        """
        给数组指定区域的元素赋值
        :param array2: 二维数组
        :param i1: 起始行数
        :param i2: 结束行数
        :param j1: 起始列数
        :param j2: 结束列数
        :param value: 数值
        """
        if i1 < 0 or j1 < 0 or i2 >= len(array2) or j2 >= len(array2[0]):
            raise IndexError("Index out of range for the provided 2D array dimensions.")

        array2[i1:i2 + 1, j1:j2 + 1] = value
        return array2

    @staticmethod
    def multiply_array_2d(array2, i1, i2, j1, j2, factor):
        """
        给数组指定区域的元素乘以系数
        :param array2: 二维数组
        :param i1: 起始行数
        :param i2: 结束行数
        :param j1: 起始列数
        :param j2: 结束列数
        :param factor: 系数
        """
        if i1 < 0 or j1 < 0 or i2 >= len(array2) or j2 >= len(array2[0]):
            raise IndexError("Index out of range for the provided 2D array dimensions.")

        array2[i1:i2 + 1, j1:j2 + 1] *= factor
        return array2

    @staticmethod
    def add_array_2d(array2, i1, i2, j1, j2, value):
        """
        给数组指定区域的元素增加数值
        :param array2: 二维数组
        :param i1: 起始行数
        :param i2: 结束行数
        :param j1: 起始列数
        :param j2: 结束列数
        :param value: 数值
        """
        if i1 < 0 or j1 < 0 or i2 >= len(array2) or j2 >= len(array2[0]):
            raise IndexError("Index out of range for the provided 2D array dimensions.")

        array2[i1:i2 + 1, j1:j2 + 1] += value
        return array2

    @staticmethod
    def assign_array_3d(array3, i1, i2, j1, j2, k1, k2, value):
        """
        给数组指定区域的元素赋值
        :param array3: 三维数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        :param value: 数值
        """
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= len(array3) or j2 >= len(array3[0]) or k2 >= len(array3[0][0]):
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        array3[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] = value
        return array3

    @staticmethod
    def multiply_array_3d(array3, i1, i2, j1, j2, k1, k2, factor):
        """
        给数组指定区域的元素乘以系数
        :param array3: 三维数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        :param factor: 系数
        """
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= len(array3) or j2 >= len(array3[0]) or k2 >= len(array3[0][0]):
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        array3[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] *= factor
        return array3

    @staticmethod
    def add_array_3d(array3, i1, i2, j1, j2, k1, k2, value):
        """
        给数组指定区域的元素增加数值
        :param array3: 三维数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        :param value: 数值
        """
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= len(array3) or j2 >= len(array3[0]) or k2 >= len(array3[0][0]):
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        array3[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] += value
        return array3

    @staticmethod
    def copy_array_3d(source_array, target_array, i1, i2, j1, j2, k1, k2):
        """
        复制源数组指定区域到目标数组
        :param source_array: 源数组
        :param target_array: 目标数组
        :param i1: 起始层数
        :param i2: 结束层数
        :param j1: 起始行数
        :param j2: 结束行数
        :param k1: 起始列数
        :param k2: 结束列数
        """
        if i1 < 0 or j1 < 0 or k1 < 0 or i2 >= len(source_array) or j2 >= len(source_array[0]) or k2 >= len(
                source_array[0][0]):
            raise IndexError("Index out of range for the provided 3D array dimensions.")

        target_array[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1] = source_array[i1:i2 + 1, j1:j2 + 1, k1:k2 + 1]
        return target_array


if __name__ == '__main__':
    _lines = ['60*0.087', '60*0.097']
    # _lines = ['1.18379        2.26189        2.61705        5.99524       10.31299',
    #           '15.17937       32.40075       33.90037      109.47345      261.72995',
    #           '238.24753      206.15742',
    #           '238.24753      206.15742      121.33305      242.47713      166.94165',
    #           '146.50681       13.91259        7.39986        5.83849        6.06996',
    #           '7.95473        8.99119']
    dt = ArrayUtils.lines_to_array_3d(_lines, 20, 3, 2, float)

    _text_format = TextFormat(number_per_line=5, decimal_places_of_zero=3, decimal_places=3)
    te = _text_format.array_3d_to_lines(dt)
    print("\n".join(te))



import unittest

from mag_tools.utils.data.value_utils import ValueUtils


class TestValueUtils(unittest.TestCase):
    def test_to_value(self):
        print(ValueUtils.to_value('12.50', float))

        # 测试整数转换
        self.assertEqual(ValueUtils.to_value("123", int), 123)
        # 测试浮点数转换
        self.assertEqual(ValueUtils.to_value("123.45", float), 123.45)
        # 测试布尔值转换
        self.assertTrue(ValueUtils.to_value("true", bool))
        self.assertFalse(ValueUtils.to_value("false", bool))
        # 测试列表转换
        self.assertEqual(ValueUtils.to_value("[1, 2, 3]", list), [1, 2, 3])
        # 测试字典转换
        self.assertEqual(ValueUtils.to_value("{'key': 'value'}", dict), {'key': 'value'})
        # 测试默认类型转换
        self.assertEqual(ValueUtils.to_value("text"), "text")

    def test_number_to_scientific(self):
        print(ValueUtils.number_to_scientific(123.50, 3))

        # 测试默认小数位数
        self.assertEqual(ValueUtils.number_to_scientific(1000000000.0), "1.000000e9")
        self.assertEqual(ValueUtils.number_to_scientific(0.000123), "1.230000e-4")

        # 测试指定小数位数
        self.assertEqual(ValueUtils.number_to_scientific(1000000000.0, 5), "1.00000e9")
        self.assertEqual(ValueUtils.number_to_scientific(0.000123, 3), "1.230e-4")

        # 测试负数
        self.assertEqual(ValueUtils.number_to_scientific(-1000000000.0), "-1.000000e9")
        self.assertEqual(ValueUtils.number_to_scientific(-0.000123), "-1.230000e-4")

        # 测试小数位数不足时补零
        self.assertEqual(ValueUtils.number_to_scientific(1.23, 5), "1.23000e0")
        self.assertEqual(ValueUtils.number_to_scientific(1.0, 5), "1.00000e0")

    def test_to_string(self):
        print(ValueUtils.to_string(125000, 6, 3))

        # 测试浮点数格式化
        self.assertEqual(ValueUtils.to_string(123.456, 2), "123.46")
        self.assertEqual(ValueUtils.to_string(123.0, 2, 2), "123.00")
        # 测试科学计数法格式化
        self.assertEqual(ValueUtils.to_string(1e9, 6, 1,True), "1.000000e9")
        # 测试整数格式化
        self.assertEqual(ValueUtils.to_string(123), "123")


import unittest


from mag_tools.utils.data.array_utils import ArrayUtils


class TestArrayUtils(unittest.TestCase):

    def test_text_to_array_1d(self):
        # 测试整数数组
        text = "1 2 3 4*2"
        expected_output = [1, 2, 3, 2, 2, 2, 2]
        result = ArrayUtils.text_to_array_1d(text, int)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        text = "1.1 2.2 3.3 2*4.4"
        expected_output = [1.1, 2.2, 3.3, 4.4, 4.4]
        result = ArrayUtils.text_to_array_1d(text, float)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        text = "true false 2*true"
        expected_output = [True, False, True, True]
        result = ArrayUtils.text_to_array_1d(text, bool)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        text = "a b c 2*d"
        expected_output = ["a", "b", "c", "d", "d"]
        result = ArrayUtils.text_to_array_1d(text, str)
        self.assertEqual(result, expected_output)

        # 测试字符型数组
        text = "1 2.2 true 2*a"
        expected_output = ["1", "2.2", "true", "a", "a"]
        result = ArrayUtils.text_to_array_1d(text, str)
        self.assertEqual(result, expected_output)

    def test_lines_to_array_1d(self):
        # 测试整数数组
        lines = [
            "1 2 3",
            "4 5 6",
            "7 8 9"
        ]
        expected_output = [1, 2, 3, 4, 5, 6, 7, 8, 9]
        result = ArrayUtils.lines_to_array_1d(lines, int)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        lines = [
            "1.1 2.2 3.3",
            "4.4 5.5 6.6",
            "7.7 8.8 9.9"
        ]
        expected_output = [1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7, 8.8, 9.9]
        result = ArrayUtils.lines_to_array_1d(lines, float)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        lines = [
            "true false true",
            "false true false"
        ]
        expected_output = [True, False, True, False, True, False]
        result = ArrayUtils.lines_to_array_1d(lines, bool)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        lines = [
            "a b c",
            "d e f",
            "g h i"
        ]
        expected_output = ["a", "b", "c", "d", "e", "f", "g", "h", "i"]
        result = ArrayUtils.lines_to_array_1d(lines, str)
        self.assertEqual(result, expected_output)

        # 测试混合类型数组
        lines = [
            "1 2.2 true",
            "2*a"
        ]
        expected_output = ['1', '2.2', 'true', 'a', 'a']
        result = ArrayUtils.lines_to_array_1d(lines, str)
        self.assertEqual(expected_output, result)

    def test_lines_to_array_2d(self):
        # 测试整数数组
        lines = [
            "1 2 3",
            "4 5 6",
            "7 8 9"
        ]
        nx, ny = 3, 3
        expected_output = [
            [1, 2, 3],
            [4, 5, 6],
            [7, 8, 9]
        ]
        result = ArrayUtils.lines_to_array_2d(lines, nx, ny, int)
        self.assertEqual(result, expected_output)

        # 测试浮点数数组
        lines = [
            "1.1 2.2 3.3",
            "4.4 5.5 6.6",
            "7.7 8.8 9.9"
        ]
        nx, ny = 3, 3
        expected_output = [
            [1.1, 2.2, 3.3],
            [4.4, 5.5, 6.6],
            [7.7, 8.8, 9.9]
        ]
        result = ArrayUtils.lines_to_array_2d(lines, nx, ny, float)
        self.assertEqual(result, expected_output)

        # 测试布尔值数组
        lines = [
            "true false true",
            "false true false"
        ]
        nx, ny = 2, 3
        expected_output = [
            [True, False, True],
            [False, True, False]
        ]
        result = ArrayUtils.lines_to_array_2d(lines, nx, ny, bool)
        self.assertEqual(result, expected_output)

        # 测试字符串数组
        lines = [
            "a b c",
            "d e f",
            "g h i"
        ]
        nx, ny = 3, 3
        expected_output = [
            ["a", "b", "c"],
            ["d", "e", "f"],
            ["g", "h", "i"]
        ]
        result = ArrayUtils.lines_to_array_2d(lines, nx, ny, str)
        self.assertEqual(result, expected_output)

        # 测试混合类型数组
        lines = [
            "1 2.2 true",
            "2*a none"
        ]
        nx, ny = 2, 3
        expected_output = [
            ["1", "2.2", "true"],
            ["a", "a", 'none']  # 由于混合类型，最后一个值可能为 None
        ]
        result = ArrayUtils.lines_to_array_2d(lines, nx, ny, str)
        self.assertEqual(result, expected_output)

    def test_lines_to_array_3d(self):
        # 测试整数数组
        block_lines = [
            "1 2 3",
            "4 5 6",
            "7 8 9",
            "10 11 12",
            "13 14 15",
            "16 17 18"
        ]
        nx, ny, nz = 2, 3, 3
        expected_output = [
            [
                [1, 2, 3],
                [4, 5, 6],
                [7, 8, 9]
            ],
            [
                [10, 11, 12],
                [13, 14, 15],
                [16, 17, 18]
            ]
        ]
        result = ArrayUtils.lines_to_array_3d(block_lines, nx, ny, nz, int)
        self.assertEqual(expected_output, result)

        # 测试浮点数数组
        block_lines = [
            "1.1 2.2 3.3",
            "4.4 5.5 6.6",
            "7.7 8.8 9.9",
            "10.1 11.2 12.3",
            "13.4 14.5 15.6",
            "16.7 17.8 18.9"
        ]
        nx, ny, nz = 2, 3, 3
        expected_output = [
            [
                [1.1, 2.2, 3.3],
                [4.4, 5.5, 6.6],
                [7.7, 8.8, 9.9]
            ],
            [
                [10.1, 11.2, 12.3],
                [13.4, 14.5, 15.6],
                [16.7, 17.8, 18.9]
            ]
        ]
        result = ArrayUtils.lines_to_array_3d(block_lines, nx, ny, nz, float)
        self.assertEqual(expected_output, result)

        # 测试布尔值数组
        block_lines = [
            "true false true",
            "false true false",
            "true true false",
            "false false true",
            "true false true",
            "false true false"
        ]
        nx, ny, nz = 2, 3, 3
        expected_output = [
            [
                [True, False, True],
                [False, True, False],
                [True, True, False]
            ],
            [
                [False, False, True],
                [True, False, True],
                [False, True, False]
            ]
        ]
        result = ArrayUtils.lines_to_array_3d(block_lines, nx, ny, nz, bool)
        self.assertEqual(expected_output, result)

        # 测试字符串数组
        block_lines = [
            "a b c",
            "d e f",
            "g h i",
            "j k l",
            "m n o",
            "p q r"
        ]
        nx, ny, nz = 2, 3, 3
        expected_output = [
            [
                ["a", "b", "c"],
                ["d", "e", "f"],
                ["g", "h", "i"]
            ],
            [
                ["j", "k", "l"],
                ["m", "n", "o"],
                ["p", "q", "r"]
            ]
        ]
        result = ArrayUtils.lines_to_array_3d(block_lines, nx, ny, nz, str)
        self.assertEqual(expected_output, result)


if __name__ == '__main__':
    unittest.main()







import json
from dataclasses import asdict, is_dataclass
from datetime import date, datetime
from decimal import Decimal
from enum import Enum
from typing import Any, Dict, List, Optional, Type, TypeVar

from anytree import Node

from mag_tools.bean.easy_map import EasyMap
from mag_tools.bean.easy_map import K, V
from mag_tools.bean.results import Results
from mag_tools.jsonparser.json_encoder import JsonEncoder
from mag_tools.model.service_status import ServiceStatus
from mag_tools.utils.data.map_utils import MapUtils
from mag_tools.utils.data.value_utils import ValueUtils

T = TypeVar('T')
E = TypeVar('E')

class JsonParser:
    @staticmethod
    def to_string(json_src: str) -> str:
        return json.loads(json_src)

    @staticmethod
    def to_decimal(json_src: str) -> Optional[Decimal]:
        if not json_src:
            return None
        return Decimal(json.loads(json_src))

    @staticmethod
    def to_float(json_src: str) -> Optional[float]:
        if not json_src:
            return None
        return float(json.loads(json_src))

    @staticmethod
    def to_int(json_src: str) -> Optional[int]:
        if not json_src:
            return None
        return int(json.loads(json_src))

    @staticmethod
    def to_datetime(json_src: str) -> Optional[datetime]:
        if not json_src:
            return None
        return datetime.strptime(json.loads(json_src), '%Y-%m-%dT%H:%M:%S.%fZ')

    @staticmethod
    def to_bool(json_src: str) -> bool:
        json_str = JsonParser.to_string(json_src).lower()
        return json_str in ("y", "yes", "true", "t", "1")

    @staticmethod
    def to_enum(json_src: str, enum_type: Optional[Type[E]]) -> Optional[E]:
        if not json_src or not issubclass(enum_type, Enum):
            return None
        enum_str = JsonParser.to_string(json_src)
        for member in enum_type:
            if member.name == enum_str or member.value == enum_str:
                return member
        return None

    @staticmethod
    def to_bean(json_src: str, clazz: Optional[Type[T]]) -> Optional[T]:
        if not json_src or not clazz:
            return None
        return json.loads(json_src, object_hook=lambda d: clazz(**d))

    @staticmethod
    def to_results(json_src: str, clazz: Optional[Type[T]]) -> Results[T]:
        if not json_src or not clazz:
            return Results()

        try:
            results_map = json.loads(json_src)
            results_map = MapUtils.keys_to_underline(results_map)

            data = list()
            for item in results_map.get('data', []):
                if clazz == bool:
                    data.append(bool(item))
                elif clazz == int:
                    data.append(int(item))
                elif clazz == float:
                    data.append(float(item))
                elif clazz == str:
                    data.append(str(item))
                elif clazz == datetime:
                    data.append(datetime.fromisoformat(item))
                else:
                    data.append(clazz(**item))

            result = Results(
                code=ServiceStatus.of_code(results_map.get('code')) if results_map.get('code') else None,
                message=results_map.get('message'),
                data=data,
                total_count=results_map.get('total_count'),
                timestamp=datetime.fromisoformat(results_map.get('timestamp')) if results_map.get('timestamp') else datetime.now()
            )
            return result
        except (json.JSONDecodeError, TypeError) as e:
            return Results.fail(str(e))

    @staticmethod
    def to_list(json_src: str, clazz: Optional[Type[T]]) -> List[T]:
        if not json_src or not clazz:
            return []
        return json.loads(json_src, object_hook=lambda d: clazz(**d))

    @staticmethod
    def to_map(json_src: str, key_cls: Optional[Type[K]], value_cls: Optional[Type[V]]) -> Dict[K, V]:
        if not json_src or not key_cls or not value_cls:
            return {}

        data_map = json.loads(json_src)
        return {
            ValueUtils.to_value(str(key), key_cls): ValueUtils.to_value(str(value), value_cls) if not isinstance(
                value, dict) else value_cls(
                **value) for key, value in data_map.items()
        }

    @staticmethod
    def to_easy_map(json_src: str, value_cls: Optional[Type[V]]) -> EasyMap[str, V]:
        return EasyMap(JsonParser.to_map(json_src, str, value_cls))


    @staticmethod
    def to_tree(json_src: str, clazz: Optional[Type[T]]) -> Node:
        if not json_src or not clazz:
            return Node("root")

        data = json.loads(json_src)

        def create_node(data_, parent=None):
            if isinstance(data_, dict):
                name = data_.get('name', 'root')
                data_object = clazz(**data_) if clazz != str else name
                node = Node(name, parent=parent, data=data_object)
                for child_data in data_.get('children', []):
                    create_node(child_data, parent=node)
            else:
                node = Node(str(data_), parent=parent)
            return node

        tree = create_node(data)
        return tree

    @staticmethod
    def from_bean(obj: Any) -> str:
        if isinstance(obj, (int, float, Decimal, bool, str)):
            return json.dumps(obj) if obj else "null"
        elif isinstance(obj, (datetime, date)):
            return obj.isoformat() if obj else "null"
        elif isinstance(obj, list):
            obj = [asdict(o) if is_dataclass(o) else o for o in obj]
            return json.dumps(obj, cls=JsonEncoder) if obj else "[]"
        elif isinstance(obj, dict):
            obj = {k: asdict(v) if is_dataclass(v) else v for k, v in obj.items()}
            return json.dumps(obj, cls=JsonEncoder) if obj else "{}"
        elif hasattr(obj, 'to_dict'):
            return json.dumps(obj.to_dict(), cls=JsonEncoder) if obj else None
        else:
            return json.dumps(obj, cls=JsonEncoder) if obj else "null"

    @staticmethod
    def from_results(obj: Results) -> str:
        json_str = JsonParser.from_bean(obj)
        results_map = json.loads(json_str)
        results_map = MapUtils.keys_to_hump(results_map)
        return json.dumps(results_map, cls=JsonEncoder) if results_map else "null"

    @staticmethod
    def to_json_by_type(obj: Any) -> Optional[str]:
        if obj is None:
            return None
        return json.dumps(obj, default=lambda o: o.__dict__, sort_keys=True, indent=4)


from mag_tools.model.justify_type import JustifyType

from mag_tools.utils.data.string_format import StringFormat
import unittest


class TestStringFormat(unittest.TestCase):
    def test_pad_string(self):
        # 测试右对齐
        self.assertEqual(StringFormat.pad_string("12.123", 10), "    12.123")
        # 测试居中对齐
        self.assertEqual(StringFormat.pad_string("12.123", 10, JustifyType.CENTER), "  12.123  ")
        # 测试左对齐
        self.assertEqual(StringFormat.pad_string("12.123", 10, JustifyType.LEFT), "12.123    ")
        # 测试字符串长度大于目标长度
        self.assertEqual(StringFormat.pad_string("12.123"), "12.123")

    def test_pad_text(self):
        # 测试多个字符串右对齐
        strings = ["test1", "test2", "test3"]
        expected_output = "     test1     test2     test3"
        self.assertEqual(expected_output, StringFormat.pad_text(strings, 10)[0])
        # 测试多个字符串居中对齐
        expected_output = " test1   test2   test3  "
        self.assertEqual(expected_output, StringFormat.pad_text(strings, 8, JustifyType.CENTER)[0])
        # 测试多个字符串左对齐
        expected_output = "test1 test2 test3"
        self.assertEqual(expected_output, StringFormat.pad_text(strings, 5, JustifyType.LEFT)[0])
        # 测试pad_len不设置
        expected_output = "test1 test2"
        self.assertEqual(expected_output, StringFormat.pad_text(strings, None, JustifyType.LEFT, 2)[0])

    def test_format(self):
        print(StringFormat.format("{random_text(5)}"))
        print(StringFormat.format("{random_string(3)}"))
        print(StringFormat.format("{random_chinese(12)}"))
        print(StringFormat.format("{random_number(4)}"))
        print(StringFormat.format("{random_date(2024,%Y-%m-%d)}"))
        print(StringFormat.format("{today(%Y%m%d)}"))
        print(StringFormat.format("{current(%Y-%m-%d %H:%M:%S)}"))
        print(StringFormat.format("{random_int(20, 30)}"))


if __name__ == '__main__':
    unittest.main()



import subprocess
import wmi
from dataclasses import dataclass, field
from typing import Optional

from mag_tools.bean.base_data import BaseData
from mag_tools.bean.sys.operate_system import OperateSystem
from mag_tools.utils.security.digest import Digest


@dataclass
class Motherboard(BaseData):
    computer_id: Optional[str] = field(default=None, metadata={"description": "所属计算机的标识"})
    manufacturer: Optional[str] = field(default=None, metadata={"description": "主板制造商"})
    product: Optional[str] = field(default=None, metadata={"description": "主板产品型号"})
    serial_number: Optional[str] = field(default=None, metadata={"description": "主板序列号"})
    version: Optional[str] = field(default=None, metadata={"description": "主板版本"})

    @property
    def hash(self):
        return Digest.md5(f'{self.serial_number}{self.manufacturer}{self.product}{self.version}')

    @classmethod
    def get_info(cls):
        """
        获取主板信息，根据操作系统选择相应的方法
        """
        if OperateSystem.is_windows():
            return cls.__get_from_windows()
        elif OperateSystem.is_linux():
            return cls.__get_from_linux()

    @classmethod
    def __get_from_windows(cls):
        """
        从 Windows 系统获取主板信息
        """
        c = wmi.WMI()
        motherboard_info = c.Win32_BaseBoard()[0]
        return cls(
            manufacturer=motherboard_info.Manufacturer,
            product=motherboard_info.Product,
            serial_number=motherboard_info.SerialNumber,
            version=motherboard_info.Version
        )

    @classmethod
    def __get_from_linux(cls):
        """
        从 Linux 系统获取主板信息
        """
        try:
            result = subprocess.run(["sudo", "dmidecode", "-t", "baseboard"], capture_output=True, text=True, check=True)
            lines = result.stdout.split("\n")
            info = {}
            for line in lines:
                if line.startswith("\tManufacturer:"):
                    info["manufacturer"] = line.split(":")[1].strip()
                elif line.startswith("\tProduct Name:"):
                    info["product"] = line.split(":")[1].strip()
                elif line.startswith("\tSerial Number:"):
                    info["serial_number"] = line.split(":")[1].strip()
                elif line.startswith("\tVersion:"):
                    info["version"] = line.split(":")[1].strip()
            return cls(
                manufacturer=info.get("manufacturer"),
                product=info.get("product"),
                serial_number=info.get("serial_number"),
                version=info.get("version")
            )
        except subprocess.CalledProcessError as e:
            print(f"Error getting motherboard information: {e}")
            return cls()

if __name__ == '__main__':
    info_ = Motherboard.get_info()
    print(info_)